searchData={"items":[{"type":"module","title":"Corsa","doc":"`Corsa` is a library to write executable code contracts in Elixir.","ref":"Corsa.html"},{"type":"module","title":"Installation - Corsa","doc":"The package can be installed by adding `corsa` to your list of dependencies in `mix.exs`:\n\n    def deps do\n      [\n        {:corsa, \"~> 0.1.1\"}\n      ]\n    end","ref":"Corsa.html#module-installation"},{"type":"module","title":"Contracts - Corsa","doc":"Corsa contracts are written using normal pre- and post-condition contracts familiar from other\nprogramming languages and libraries such as Eiffel. To use the library in a module we must first\ninsert a\n\n    use Corsa\n\nstatement in the module. Suppose next that we have implemented a function `list_to_nat` which\nconverts an integer represented as a list of small numbers (between 0 and 9) to an integer:\n\nSuppose next that we have implemented a function `list_to_nat` which converts an integer\nrepresented as a list of small numbers (between 0 and 9) to an integer. We can require that the\nargument to `list_to_nat` is correct by specifying a `@pre` contract:\n\n    iex> defmodule Example do\n    ...>   use Corsa\n    ...>   @pre list_to_nat(l), do: is_decimal_list(l)\n    ...>   @post list_to_nat(l), do: is_integer(result)\n    ...>   def list_to_nat(list), do: list_to_nat(list, 0)\n    ...>   defp list_to_nat([], result), do: result\n    ...>   defp list_to_nat([ n| rest ], acc) do\n    ...>     list_to_nat(rest ,10*acc+n)\n    ...>   end\n    ...>   defp is_decimal_list([n|rest]) do\n    ...>     is_decimal(n) and (n > 0)\n    ...>     and Enum.all?(rest,&is_decimal/1)\n    ...>   end\n    ...>   defp is_decimal_list(_), do: false\n    ...>   defp is_decimal(n) do\n    ...>     is_integer(n) and (n >= 0) and (n < 10)\n    ...>   end\n    ...> end\n    iex> Example.list_to_nat([1,5])\n    15\n    iex> Example.list_to_nat(5)\n    ** (Corsa.PreViolationError) @pre does not hold in call 'CorsaDoctestTest.Example.list_to_nat(5)'\n\nIf we now invoke the function with an incorrect argument by default an informative exception is\nraised.\n\nSimilarly `@post` contracts check that the value returned by a function is correct:","ref":"Corsa.html#module-contracts"},{"type":"module","title":"Assertions - Corsa","doc":"Assertions can be stated using the `@assert` construct:\n\n    iex> defmodule AssertExample do\n    ...>   use Corsa\n    ...>   def f(y) do\n    ...>     x = g(y)\n    ...>     @assert x > 0\n    ...>     x + 10\n    ...>   end\n    ...>   defp g(x) do x end\n    ...> end\n    iex> AssertExample.f(10)\n    20\n    iex> AssertExample.f(-10)\n    ** (Corsa.AssertViolationError) @assert does not hold in expression 'x > 0' with values left '-10' and right '0'","ref":"Corsa.html#module-assertions"},{"type":"module","title":"Termination Guarantees - Corsa","doc":"Corsa can check whether recursive functions eventually terminate, through specifying a measure\nusing the `@decreases` construct:\n\n    iex> defmodule DecreasesExample1 do\n    ...>   use Corsa\n    ...>   @decreases sum(l), do: length(l)\n    ...>   def sum([]), do: 0\n    ...>   def sum([n|rest]), do: n+sum(rest)\n    ...> end\n    iex> DecreasesExample1.sum([1,2,3])\n    6\n\nThe `@decreases` annotation specifies a function, that when applied to the arguments of the called\nfunction, calculates a measure (e.g., an integer). Corsa checks that measure when a function is\ninvoked is strictly greater than the measure in recursive calls.  In the example above, the\nmeasure used is the length of the least which strictly decreases in recursive calls.  The check\nwill be made even if the recursive call is made within another function.\n\nNote, however, that a decreasing measure does not necessarily guarantee termination:\n\n    defmodule DecreasesExample2 do\n      use Corsa\n      @decreases fact(n), do: n\n      def fact(1), do: 1\n      def fact(n), do: n * fact(n - 1)\n    end\n\nIf the function is invoked with `fact(-1)` then it obviously never terminates (except possibly\nwith an exception) but the measure always strictly decreases. Note you can \"fix\" the previous\ncontract by using `abs/1`:\n\n    iex> defmodule DecreasesExample3 do\n    ...>   use Corsa\n    ...>   @decreases fact(n), do: abs(n)\n    ...>   def fact(1), do: 1\n    ...>   def fact(n), do: n * fact(n - 1)\n    ...> end\n    iex> DecreasesExample3.fact(10)\n    3628800\n    iex> DecreasesExample3.fact(0)\n    ** (Corsa.DecreasesViolationError) @decreases does not hold, call 'CorsaDoctestTest.DecreasesExample3.fact(-1)' does not decrease, previous measure: '0', new measure: '1'","ref":"Corsa.html#module-termination-guarantees"},{"type":"module","title":"Time Contracts - Corsa","doc":"Time bounds for long a function application may execute can be specified with the `@within`\nconstruct:\n\n    iex> defmodule WithinExample do\n    ...>   use Corsa\n    ...>   @within f(n) do n end\n    ...>   def f(n) do Process.sleep(n + 10) end\n    ...> end\n    iex> WithinExample.f(1)\n    ** (Corsa.WithinViolationError) @within does not hold in call 'CorsaDoctestTest.WithinExample.f(1)'\n\nIf the call to f function does not return an answer within n + 10 milliseconds, an error is\nlogged.","ref":"Corsa.html#module-time-contracts"},{"type":"module","title":"Message Passing Contracts - Corsa","doc":"The `@expects` contract specifies the accepatble messages in a receive statement.\n\n    iex> defmodule ExpectsExample do\n    ...>   use Corsa\n    ...>   def counter(state) do\n    ...>     @expects v > 0, receive do\n    ...>       :inc -> counter(state + 1)\n    ...>       :dec -> counter(state - 1)\n    ...>       {:set, v} -> counter(v)\n    ...>     end\n    ...>   end\n    ...> end\n    iex> send(self(), :inc)\n    iex> send(self(), {:set, 10})\n    iex> send(self(), {:set, -1})\n    iex> ExpectsExample.counter(0)\n    ** (Corsa.ExpectsViolationError) @expects does not hold in expression 'v > 0' with values left '-1' and right '0'","ref":"Corsa.html#module-message-passing-contracts"},{"type":"module","title":"Variable Contracts - Corsa","doc":"Variable contracts allow you to define variable contracts. In Corsa, these contracts restrict the values\nthat may be assigned to a variable.\n\n\n    # iex> defmodule DeclareExample do\n    # ...>   use Corsa\n    # ...>   @declare is_integer(n) && n > 0\n    # ...>   def counter(n) do\n    # ...>     receive do\n    # ...>       :inc -> counter(n + 1)\n    # ...>       :dec -> counter(n - 1)\n    # ...>       {:set, n} -> counter(n)\n    # ...>     end\n    # ...>   end\n    # ...> end\n    # iex> send(self(), :inc)\n    # iex> send(self(), {:set, 10})\n    # iex> send(self(), {:set, -1})\n    # iex> DeclareExample.counter(0)","ref":"Corsa.html#module-variable-contracts"},{"type":"module","title":"Type Contracts - Corsa","doc":"When using `Corsa` type annotations will be used to generate runtime typechecks for your\nfunctions.\n\n    iex> defmodule TypedExample do\n    ...>   use Corsa\n    ...>   @spec id(integer()) :: integer()\n    ...>   @spec id(boolean()) :: boolean()\n    ...>   def id(x) do x end\n    ...> end\n    iex> TypedExample.id(10)\n    10\n    iex> TypedExample.id(true)\n    true\n    iex> TypedExample.id(\"hello\")\n    ** (Corsa.SpecArgViolationError) @spec does not hold in arguments in call 'CorsaDoctestTest.TypedExample.id(\"hello\")'","ref":"Corsa.html#module-type-contracts"},{"type":"module","title":"State Contracts - Corsa","doc":"iex> defmodule StateExample do\n    ...>   use Corsa, state: true\n    ...>   @init :not_started\n    ...>\n    ...>   @next start(), do: 0\n    ...>   @throws start(), do: not started?(:not_started)\n    ...>   def start() do\n    ...>     try do\n    ...>       :ets.new(:counter, [:named_table])\n    ...>       :ets.insert(:counter, {\"value\", 0})\n    ...>       :ok\n    ...>     catch\n    ...>       _, _ -> raise \"error already started\"\n    ...>     end\n    ...>   end\n    ...>\n    ...>   @next inc(), do: state + 1\n    ...>   @throws inc(), do: started?(state)\n    ...>   def inc() do\n    ...>     [{\"value\", v}] = :ets.lookup(:counter, \"value\")\n    ...>     :ets.insert(:counter, {\"value\", v + 1})\n    ...>     :ok\n    ...>   end\n    ...>\n    ...>   @throws get(), do: started?(state)\n    ...>   def get() do\n    ...>     [{\"value\", v}] = :ets.lookup(:counter, \"value\")\n    ...>     v\n    ...>   end\n    ...>   def started?(state), do: state != :not_started\n    ...> end\n    iex> StateExample.start()\n    iex> StateExample.inc()\n    iex> StateExample.inc()\n    iex> StateExample.get()\n    2\n    iex> StateExample.start()\n    ** (RuntimeError) error already started","ref":"Corsa.html#module-state-contracts"},{"type":"module","title":"Monitors - Corsa","doc":"","ref":"Corsa.html#module-monitors"},{"type":"module","title":"Configuring Corsa - Corsa","doc":"Corsa can be configured to either (the default) raise exceptions whenever a contract is violated,\nor log contract violations, or do nothing.","ref":"Corsa.html#module-configuring-corsa"},{"type":"module","title":"Logging levels - Corsa","doc":"iex> defmodule LogsExample do\n    ...>   use Corsa, logger: :error\n    ...>   @pre list_to_nat(l), do: is_decimal_list(l)\n    ...>   @post list_to_nat(l), do: is_integer(result)\n    ...>   def list_to_nat(list), do: list_to_nat(list, 0)\n    ...>   defp list_to_nat([], result), do: result\n    ...>   defp list_to_nat([ n| rest ], acc) do\n    ...>     list_to_nat(rest ,10*acc+n)\n    ...>   end\n    ...>   defp is_decimal_list([n|rest]) do\n    ...>     is_decimal(n) and (n > 0)\n    ...>     and Enum.all?(rest,&is_decimal/1)\n    ...>   end\n    ...>   defp is_decimal_list(_), do: false\n    ...>   defp is_decimal(n) do\n    ...>     is_integer(n) and (n >= 0) and (n < 10)\n    ...>   end\n    ...> end","ref":"Corsa.html#module-logging-levels"},{"type":"module","title":"Disabling contracts - Corsa","doc":"iex> defmodule DisabledContracts do\n    ...>   use Corsa, disabled: true\n    ...>   @pre f(x), do: x > 0\n    ...>   @post f(x), do: result > 0\n    ...>   def f(x), do: x\n    ...> end\n    iex> DisabledContracts.f(-1)\n    -1","ref":"Corsa.html#module-disabling-contracts"},{"type":"module","title":"Corsa.Assert","doc":"Assertions in programming are statements that check conditions. They help catch errors early and\nimprove code reliability and maintainability. In Corsa, assertions can work in two ways: by\nthrowing exceptions or logging errors.","ref":"Corsa.Assert.html"},{"type":"module","title":"Examples - Corsa.Assert","doc":"iex> defmodule Elixir.Corsa.Assert.ExampleAssert do\n    ...>   use Corsa.Assert\n    ...>   def f(x) do\n    ...>     Corsa.Assert.assert x > 0\n    ...>     :ok\n    ...>   end\n    ...> end\n    iex> Elixir.Corsa.Assert.ExampleAssert.f(10)\n    :ok\n    iex> Elixir.Corsa.Assert.ExampleAssert.f(0)\n    ** (Corsa.AssertViolationError) @assert does not hold in expression 'x > 0' with value left '0'\n\n    iex> f = fn -> {:ok, 1} end\n    iex> Corsa.Assert.assert {:ok, x} = f.()\n    iex> x\n    1\n    iex> x = 2\n    iex> Corsa.Assert.assert {:ok, ^x} = f.()\n    ** (Corsa.AssertViolationError) @assert does not hold in expression '{:ok, ^x} = f.()' with values left '{:ok, ^x}' and right '{:ok, 1}'","ref":"Corsa.Assert.html#module-examples"},{"type":"macro","title":"Corsa.Assert.assert/4","doc":"","ref":"Corsa.Assert.html#assert/4"},{"type":"module","title":"Corsa.Declare","doc":"Declare allows you to define variable contracts. In Corsa, these contracts restrict the values\nthat may be assigned to a variable.","ref":"Corsa.Declare.html"},{"type":"module","title":"Example - Corsa.Declare","doc":"iex> defmodule Elixir.Corsa.Declare.Example do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Declare\n    ...>   declare is_integer(n) && n > 0\n    ...>   def f() do\n    ...>     n = 10\n    ...>     n\n    ...>   end\n    ...>   def g(n) do\n    ...>     n\n    ...>   end\n    ...> end\n    iex> Elixir.Corsa.Declare.Example.f()\n    10\n    iex> Elixir.Corsa.Declare.Example.g(-1)\n    ** (Corsa.DeclareViolationError) @declare does not hold\n\n    iex> defmodule Elixir.Corsa.Declare.Example1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Declare\n    ...>   declare is_integer(n) && n > 0\n    ...>   def f() do\n    ...>     n = -1\n    ...>     n\n    ...>   end\n    ...> end\n    iex> Elixir.Corsa.Declare.Example1.f()\n    ** (Corsa.DeclareViolationError) @declare does not hold\n\n    iex> defmodule Elixir.Corsa.Declare.Example2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Declare\n    ...>   declare is_integer(n) && n > 0\n    ...>   def f() do\n    ...>     receive do\n    ...>       n -> n\n    ...>     end\n    ...>   end\n    ...> end\n    iex> send(self(), -10)\n    iex> Elixir.Corsa.Declare.Example2.f()\n    ** (Corsa.DeclareViolationError) @declare does not hold\n\n    iex> defmodule Elixir.Corsa.Declare.Example3 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Declare\n    ...>   declare n > 0\n    ...>   def f() do\n    ...>     receive do\n    ...>       n when is_integer(n) -> n\n    ...>     end\n    ...>   end\n    ...> end\n    iex> send(self(), -10)\n    iex> Elixir.Corsa.Declare.Example3.f()\n    ** (Corsa.DeclareViolationError) @declare does not hold","ref":"Corsa.Declare.html#module-example"},{"type":"function","title":"Corsa.Declare.bound_vars/1","doc":"","ref":"Corsa.Declare.html#bound_vars/1"},{"type":"macro","title":"Corsa.Declare.declare/1","doc":"","ref":"Corsa.Declare.html#declare/1"},{"type":"macro","title":"Errors - Corsa.Declare.declare/1","doc":"iex> defmodule Elixir.Corsa.Declare.ExampleError do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Declare\n    ...>   declare is_integer(n) && m > 0\n    ...> end\n    ** (Corsa.DeclareError) The expression in @declare should contain just one free variable","ref":"Corsa.Declare.html#declare/1-errors"},{"type":"module","title":"Corsa.Decreases","doc":"The decreases contract specifies a function, that when applied to the arguments of the called\nfunction, calculates a measure (e.g., an integer). Corsa checks that measure when a function is\ninvoked is strictly greater than the measure in recursive calls.","ref":"Corsa.Decreases.html"},{"type":"module","title":"Example - Corsa.Decreases","doc":"iex> defmodule Elixir.Corsa.Decreases.Example do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Decreases\n    ...>   decreases fact(n) do n end\n    ...>   def fact(0) do fact(1) end\n    ...>   def fact(n) do n * fact(n - 1) end\n    ...> end\n    iex> Elixir.Corsa.Decreases.Example.fact(10)\n    ** (Corsa.DecreasesViolationError) @decreases does not hold, call 'Corsa.Decreases.Example.fact(1)' does not decrease, previous measure: '0', new measure: '1'","ref":"Corsa.Decreases.html#module-example"},{"type":"macro","title":"Corsa.Decreases.decreases/2","doc":"","ref":"Corsa.Decreases.html#decreases/2"},{"type":"macro","title":"Errors - Corsa.Decreases.decreases/2","doc":"iex> defmodule Elixir.Corsa.Decreases.ExampleError1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Decreases\n    ...>   decreases fact(n) do n end\n    ...>   decreases fact(n) do n end\n    ...>  def fact(0) do fact(1) end\n    ...>  def fact(n) do n * fact(n - 1) end\n    ...> end\n    ** (Corsa.DecreasesError) @decreases for function 'fact/1' already defined\n\n    iex> defmodule Elixir.Corsa.Decreases.ExampleError2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Decreases\n    ...>   decreases fact(n, n) do n end\n    ...>   def fact(0) do fact(1) end\n    ...>   def fact(n) do n * fact(n - 1) end\n    ...> end\n    ** (Corsa.DecreasesError) arguments in @decreases should contain different names\n\n    iex> defmodule Elixir.Corsa.Decreases.ExampleError3 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Decreases\n    ...>   decreases fact(_) do n end\n    ...>   def fact(0) do fact(1) end\n    ...>   def fact(n) do n * fact(n - 1) end\n    ...> end\n    ** (Corsa.DecreasesError) arguments in @decreases cannot be ignored with _","ref":"Corsa.Decreases.html#decreases/2-errors"},{"type":"module","title":"Corsa.Expects","doc":"The expect specifies the acceptable messages in a receive statement.","ref":"Corsa.Expects.html"},{"type":"module","title":"Example - Corsa.Expects","doc":"iex> defmodule Elixir.Corsa.Expects.Example do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Expects\n    ...>\n    ...>   def counter(state) do\n    ...>     expects [v > 0, receive do\n    ...>       :inc -> counter(state + 1)\n    ...>       :dec -> counter(state - 1)\n    ...>       {:set, v} -> counter(v)\n    ...>     end]\n    ...>   end\n    ...> end\n    iex> send(self(), :inc)\n    iex> send(self(), {:set, 10})\n    iex> send(self(), {:set, -1})\n    iex> Elixir.Corsa.Expects.Example.counter(0)\n    ** (Corsa.ExpectsViolationError) @expects does not hold in expression 'v > 0' with values left '-1' and right '0'","ref":"Corsa.Expects.html#module-example"},{"type":"macro","title":"Corsa.Expects.expects/1","doc":"","ref":"Corsa.Expects.html#expects/1"},{"type":"macro","title":"Errors - Corsa.Expects.expects/1","doc":"iex> defmodule Elixir.Corsa.Expects.ExampleError1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Expects\n    ...>\n    ...>   def f(state) do\n    ...>     expects [error, receive do end]\n    ...>   end\n    ...> end\n    ** (Corsa.ExpectsError) syntax error in @expects\n\n    iex> defmodule Elixir.Corsa.Expects.ExampleError2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Expects\n    ...>\n    ...>   def f(state) do\n    ...>     expects [v > 0, error do end]\n    ...>   end\n    ...> end\n    ** (Corsa.ExpectsError) syntax error in @expects","ref":"Corsa.Expects.html#expects/1-errors"},{"type":"module","title":"Corsa.Post","doc":"Postconditions are specifications which must be fulfilled after the execution of a function,\nthereby certifying that the anticipated output criteria have been satisfied. In Corsa,\npostconditions are evaluated at runtime. If a postcondition is not satisfied, either an exception\nor a log is generated.","ref":"Corsa.Post.html"},{"type":"module","title":"Example - Corsa.Post","doc":"iex> defmodule Elixir.Corsa.Post.Example do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Post\n    ...>   post f(x, y) do result > x end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    iex> Elixir.Corsa.Post.Example.f(2, 1)\n    3\n    iex> Elixir.Corsa.Post.Example.f(1, -1)\n    ** (Corsa.PostViolationError) @post does not hold in call 'Corsa.Post.Example.f(1, -1)' with result '0'","ref":"Corsa.Post.html#module-example"},{"type":"macro","title":"Corsa.Post.post/2","doc":"","ref":"Corsa.Post.html#post/2"},{"type":"macro","title":"Errors - Corsa.Post.post/2","doc":"iex> defmodule Elixir.Corsa.Post.ExampleError1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Post\n    ...>   post f(x, y) do x > y end\n    ...>   post f(x, y) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) @post for function 'f/2' already defined\n\n    iex> defmodule Elixir.Corsa.Post.ExampleError2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Post\n    ...>   post f(x, x) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) arguments in @post should contain different names\n\n    iex> defmodule Elixir.Corsa.Post.ExampleError3 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Post\n    ...>   post f(x, _) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) arguments in @post cannot be ignored with _\n\n    iex> defmodule Elixir.Corsa.Post.ExampleError4 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Post\n    ...>   post f(x, result) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) result cannot be an argument in @post","ref":"Corsa.Post.html#post/2-errors"},{"type":"module","title":"Corsa.Pre","doc":"Preconditions are requirements that must be met before executing a function. It ensures correct\nbehavior and avoids potential errors by specifying conditions for input parameters. In Corsa,\npreconditions are checked during runtime. If a precondition is not met, an exception or a log is\nthrown.","ref":"Corsa.Pre.html"},{"type":"module","title":"Examples - Corsa.Pre","doc":"iex> defmodule Elixir.Corsa.Pre.Example do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Pre\n    ...>   pre f(x, y) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    iex> Elixir.Corsa.Pre.Example.f(2, 1)\n    3\n    iex> Elixir.Corsa.Pre.Example.f(1, 2)\n    ** (Corsa.PreViolationError) @pre does not hold in call 'Corsa.Pre.Example.f(1, 2)'","ref":"Corsa.Pre.html#module-examples"},{"type":"macro","title":"Corsa.Pre.pre/2","doc":"","ref":"Corsa.Pre.html#pre/2"},{"type":"macro","title":"Errors - Corsa.Pre.pre/2","doc":"iex> defmodule Elixir.Corsa.Pre.ExampleError1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Pre\n    ...>   pre f(x, y) do x > y end\n    ...>   pre f(x, y) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PreError) @pre for function 'f/2' already defined\n\n    iex> defmodule Elixir.Corsa.Pre.ExampleError2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Pre\n    ...>   pre f(x, x) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PreError) arguments in @pre should contain different names\n\n    iex> defmodule Elixir.Corsa.Pre.ExampleError3 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Pre\n    ...>   pre f(x, _) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PreError) arguments in @pre cannot be ignored with _","ref":"Corsa.Pre.html#pre/2-errors"},{"type":"module","title":"Corsa.Spec","doc":"SpecArgs contract, also known as type requirements, must be met before executing a function. In\nCorsa, they are checked during runtime and just prior to the annotated function's execution.","ref":"Corsa.Spec.html"},{"type":"module","title":"Example - Corsa.Spec","doc":"iex> defmodule Elixir.Corsa.Spec.Example do\n...>   use Corsa.Type\n...>   use Corsa.Assert\n...>   use Corsa.Spec\n...>   spec f(arg1 :: integer(), arg2 :: boolean()), :ok\n...>   def f(1, false), do: :error\n...>   def f(_arg1, _arg2), do: :ok\n...> end\niex> Elixir.Corsa.Spec.Example.f(1, true)\niex> Elixir.Corsa.Spec.Example.f(1, false)\n** (Corsa.SpecResultViolationError) @spec does not hold in call 'Corsa.Spec.Example.f(1, true)' with result ':ok'","ref":"Corsa.Spec.html#module-example"},{"type":"macro","title":"Corsa.Spec.spec/2","doc":"","ref":"Corsa.Spec.html#spec/2"},{"type":"macro","title":"Errors - Corsa.Spec.spec/2","doc":"iex> defmodule Elixir.Corsa.Spec.ExampleError do\n...>   use Corsa.Type\n...>   use Corsa.Assert\n...>   use Corsa.Spec\n...>   spec f(arg1 :: integer(), arg1 :: boolean()), boolean()\n...>   def f(_arg1, _arg2), do: :ok\n...> end\n** (Corsa.SpecError) arguments in @spec should contain different names","ref":"Corsa.Spec.html#spec/2-errors"},{"type":"module","title":"Corsa.StateM","doc":"TODO","ref":"Corsa.StateM.html"},{"type":"module","title":"Corsa.StateM.Init","doc":"The `init` annotation is used to establish the initial state of the state machine.","ref":"Corsa.StateM.Init.html"},{"type":"macro","title":"Corsa.StateM.Init.init/1","doc":"","ref":"Corsa.StateM.Init.html#init/1"},{"type":"macro","title":"Errors - Corsa.StateM.Init.init/1","doc":"iex> defmodule Elixir.Corsa.StateM.Init.ExampleInitError do\n    ...>   use Corsa.StateM, state: true\n    ...>   init 0\n    ...>   init 0\n    ...> end\n    ** (Corsa.StateM.InitError) @init already defined","ref":"Corsa.StateM.Init.html#init/1-errors"},{"type":"module","title":"Corsa.StateM.Model","doc":"This module generates a state machine model using contract information.","ref":"Corsa.StateM.Model.html"},{"type":"module","title":"Examples - Corsa.StateM.Model","doc":"iex> defmodule Elixir.Corsa.StateM.Model.Example1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Init\n    ...>   use Corsa.StateM.Post\n    ...>   use Corsa.StateM.Next\n    ...>   use Corsa.StateM.Model\n    ...>\n    ...>   init 0\n    ...>   next f(), do: state + 1\n    ...>   post f(), do: result == state\n    ...>   def f() do 0 end\n    ...> end\n    iex> Elixir.Corsa.StateM.Model.Example1.init()\n    0\n    iex> Elixir.Corsa.StateM.Model.Example1.cmds()\n    [f: 0]\n    iex> Elixir.Corsa.StateM.Model.Example1.next(0, {:call, Elixir.Corsa.StateM.Model.Example1, :f, []}, 0)\n    1\n    iex> Elixir.Corsa.StateM.Model.Example1.post(1, {:call, Elixir.Corsa.StateM.Model.Example1, :f, []}, 0)","ref":"Corsa.StateM.Model.html#module-examples"},{"type":"module","title":"Corsa.StateM.Monitor","doc":"Macros in this module generate the necessary intrumentation to monitor an Elixir process.\n\nIt also contains the main API to interact with `Corsa` monitors.","ref":"Corsa.StateM.Monitor.html"},{"type":"module","title":"Examples - Corsa.StateM.Monitor","doc":"iex> defmodule Elixir.Corsa.StateM.Monitor.Example1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM, state: true\n    ...>   use Corsa.StateM.Monitor\n    ...>   use Corsa.StateM.Model\n    ...>\n    ...>   init 0\n    ...>   next f(), do: state + 1\n    ...>   post f(), do: result == state\n    ...>   def f() do 0 end\n    ...> end\n    iex> Elixir.Corsa.StateM.Monitor.Example1.f()\n    iex> Elixir.Corsa.StateM.Monitor.Example1.f()\n    ** (Corsa.PostViolationError) @post does not hold in call 'Corsa.StateM.Monitor.Example1.f()' with result '0' in state '1'","ref":"Corsa.StateM.Monitor.html#module-examples"},{"type":"function","title":"Corsa.StateM.Monitor.finish_monitored_call/4","doc":"TODO","ref":"Corsa.StateM.Monitor.html#finish_monitored_call/4"},{"type":"function","title":"Corsa.StateM.Monitor.sleep/2","doc":"","ref":"Corsa.StateM.Monitor.html#sleep/2"},{"type":"function","title":"Corsa.StateM.Monitor.start_monitored_call/2","doc":"TODO","ref":"Corsa.StateM.Monitor.html#start_monitored_call/2"},{"type":"function","title":"Corsa.StateM.Monitor.stop/2","doc":"","ref":"Corsa.StateM.Monitor.html#stop/2"},{"type":"module","title":"Corsa.StateM.MonitorGenServer","doc":"TODO","ref":"Corsa.StateM.MonitorGenServer.html"},{"type":"function","title":"Corsa.StateM.MonitorGenServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Corsa.StateM.MonitorGenServer.html#child_spec/1"},{"type":"function","title":"Corsa.StateM.MonitorGenServer.start_link/2","doc":"","ref":"Corsa.StateM.MonitorGenServer.html#start_link/2"},{"type":"module","title":"Corsa.StateM.Next","doc":"The `next` annotation is utilized to delineate the transition function of the state machine.","ref":"Corsa.StateM.Next.html"},{"type":"macro","title":"Corsa.StateM.Next.next/2","doc":"","ref":"Corsa.StateM.Next.html#next/2"},{"type":"macro","title":"Errors - Corsa.StateM.Next.next/2","doc":"iex> defmodule Elixir.Corsa.StateM.Next.ExampleNextError1 do\n    ...>   use Corsa.StateM.Next\n    ...>   next f(x, y) do x + y end\n    ...>   next f(x, y) do x + y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.StateM.NextError) @next for function 'f/2' already defined\n\n    iex> defmodule Elixir.Corsa.StateM.Next.ExampleNextError2 do\n    ...>   use Corsa.StateM.Next\n    ...>   next f(x, x) do x + y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.StateM.NextError) arguments in @next should contain different names\n\n    iex> defmodule Elixir.Corsa.StateM.Next.ExampleNextError3 do\n    ...>   use Corsa.StateM.Next\n    ...>   next f(x, _) do x + y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.StateM.NextError) arguments in @next cannot be ignored with _\n\n    iex> defmodule Elixir.Corsa.StateM.Next.ExampleNextError4 do\n    ...>   use Corsa.StateM, state: true\n    ...>   next f(x, result) do x + y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.StateM.NextError) result cannot be an argument in @next","ref":"Corsa.StateM.Next.html#next/2-errors"},{"type":"module","title":"Corsa.StateM.OnlineMonitorGenServer","doc":"TODO","ref":"Corsa.StateM.OnlineMonitorGenServer.html"},{"type":"function","title":"Corsa.StateM.OnlineMonitorGenServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Corsa.StateM.OnlineMonitorGenServer.html#child_spec/1"},{"type":"function","title":"Corsa.StateM.OnlineMonitorGenServer.start_link/2","doc":"","ref":"Corsa.StateM.OnlineMonitorGenServer.html#start_link/2"},{"type":"module","title":"Corsa.StateM.Post","doc":"Postconditions are essential conditions that must be fulfilled following the execution of a\nfunction, thereby assuring that the anticipated output conditions have been met. In Corsa,\npostconditions are evaluated during runtime. If a postcondition isn't guaranteed, an exception or\nlog is generated. Within the definition of a state machine, postconditions are able to access the\nstate.","ref":"Corsa.StateM.Post.html"},{"type":"macro","title":"Corsa.StateM.Post.post/2","doc":"","ref":"Corsa.StateM.Post.html#post/2"},{"type":"macro","title":"Errors - Corsa.StateM.Post.post/2","doc":"iex> defmodule Elixir.Corsa.StateM.Post.ExampleError1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Post\n    ...>   post f(x, y) do x > y end\n    ...>   post f(x, y) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) @post for function 'f/2' already defined\n\n    iex> defmodule Elixir.Corsa.StateM.Post.ExampleError2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Post\n    ...>   post f(x, x) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) arguments in @post should contain different names\n\n    iex> defmodule Elixir.Corsa.StateM.Post.ExampleError3 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Post\n    ...>   post f(x, _) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) arguments in @post cannot be ignored with _\n\n    iex> defmodule Elixir.Corsa.StateM.Post.ExampleError4 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Post\n    ...>   post f(x, result) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) result cannot be an argument in @post\n\n    iex> defmodule Elixir.Corsa.StateM.Post.ExampleError4 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Post\n    ...>   post f(x, state) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.PostError) result cannot be an argument in @post","ref":"Corsa.StateM.Post.html#post/2-errors"},{"type":"module","title":"Corsa.StateM.Throws","doc":"`throws` verifies if a thrown exception is anticipated. When a state machine is utilized, the\ncontract has the ability to access the state.","ref":"Corsa.StateM.Throws.html"},{"type":"macro","title":"Corsa.StateM.Throws.throws/2","doc":"","ref":"Corsa.StateM.Throws.html#throws/2"},{"type":"macro","title":"Errors - Corsa.StateM.Throws.throws/2","doc":"iex> defmodule Elixir.Corsa.StateM.Throws.ExampleError1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Throws\n    ...>   throws f(x, y) do x > y end\n    ...>   throws f(x, y) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) @throws for function 'f/2' already defined\n\n    iex> defmodule Elixir.Corsa.StateM.Throws.ExampleError2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Throws\n    ...>   throws f(x, x) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) arguments in @throws should contain different names\n\n    iex> defmodule Elixir.Corsa.StateM.Throws.ExampleError3 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Throws\n    ...>   throws f(x, _) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) arguments in @throws cannot be ignored with _\n\n    iex> defmodule Elixir.Corsa.StateM.Throws.ExampleError4 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Throws\n    ...>   throws f(kind, x) do x > 0 end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) 'kind' cannot be an argument in @throws\n\n    iex> defmodule Elixir.Corsa.StateM.Throws.ExampleError5 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.StateM.Throws\n    ...>   throws f(value, x) do value < 0 end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) 'value' cannot be an argument in @throws","ref":"Corsa.StateM.Throws.html#throws/2-errors"},{"type":"module","title":"Corsa.Throws","doc":"Throws checks if the exception thrown by the annotated function is expected.","ref":"Corsa.Throws.html"},{"type":"module","title":"Example - Corsa.Throws","doc":"iex> defmodule Elixir.Corsa.Throws.Example do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Throws\n    ...>   throws f(x, y) do x > y end\n    ...>   def f(x, y) when x     def f(x, y) do x + y end\n    ...> end\n    iex> Elixir.Corsa.Throws.Example.f(2, 1)\n    3\n    iex> Elixir.Corsa.Throws.Example.f(1, 2)\n    ** (Corsa.ThrowsViolationError) @throws does not hold in call 'Corsa.Throws.Example.f(1, 2)'","ref":"Corsa.Throws.html#module-example"},{"type":"macro","title":"Corsa.Throws.throws/2","doc":"","ref":"Corsa.Throws.html#throws/2"},{"type":"macro","title":"Errors - Corsa.Throws.throws/2","doc":"iex> defmodule Elixir.Corsa.Throws.ExampleError1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Throws\n    ...>   throws f(x, y) do x > y end\n    ...>   throws f(x, y) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) @throws for function 'f/2' already defined\n\n    iex> defmodule Elixir.Corsa.Throws.ExampleError2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Throws\n    ...>   throws f(x, x) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) arguments in @throws should contain different names\n\n    iex> defmodule Elixir.Corsa.Throws.ExampleError3 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Throws\n    ...>   throws f(x, _) do x > y end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) arguments in @throws cannot be ignored with _\n\n    iex> defmodule Elixir.Corsa.Throws.ExampleError4 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Throws\n    ...>   throws f(kind, x) do x > 0 end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) 'kind' cannot be an argument in @throws\n\n    iex> defmodule Elixir.Corsa.Throws.ExampleError5 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Throws\n    ...>   throws f(value, x) do value < 0 end\n    ...>   def f(x, y) do x + y end\n    ...> end\n    ** (Corsa.ThrowsError) 'value' cannot be an argument in @throws","ref":"Corsa.Throws.html#throws/2-errors"},{"type":"module","title":"Corsa.Type","doc":"","ref":"Corsa.Type.html"},{"type":"function","title":"Corsa.Type._collect_types/2","doc":"This functionextracts types on dynamically loaded modules that use Corsa and stores it in\nCorsa.Type cache.","ref":"Corsa.Type.html#_collect_types/2"},{"type":"macro","title":"Corsa.Type.<~>/2","doc":"","ref":"Corsa.Type.html#%3C~%3E/2"},{"type":"macro","title":"Corsa.Type.conforms/2","doc":"# Basic Types","ref":"Corsa.Type.html#conforms/2"},{"type":"macro","title":"Corsa.Type.conforms?/2","doc":"","ref":"Corsa.Type.html#conforms?/2"},{"type":"function","title":"Corsa.Type.conforms_type/3","doc":"","ref":"Corsa.Type.html#conforms_type/3"},{"type":"function","title":"Corsa.Type.conforms_type?/3","doc":"","ref":"Corsa.Type.html#conforms_type?/3"},{"type":"function","title":"Corsa.Type.fetch_protocols/0","doc":"","ref":"Corsa.Type.html#fetch_protocols/0"},{"type":"function","title":"Corsa.Type.fetch_type/2","doc":"","ref":"Corsa.Type.html#fetch_type/2"},{"type":"function","title":"Corsa.Type.fetch_types/1","doc":"","ref":"Corsa.Type.html#fetch_types/1"},{"type":"function","title":"Corsa.Type.solve_alias/2","doc":"","ref":"Corsa.Type.html#solve_alias/2"},{"type":"module","title":"Corsa.Within","doc":"Within contracts, there are time bounds to function executions. In Corsa, when an annotated\nfunction does not execute within the time bound specified by the contract, a runtime error is\nthrown.","ref":"Corsa.Within.html"},{"type":"module","title":"Example - Corsa.Within","doc":"iex> defmodule Elixir.Corsa.Within.Example do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Within\n    ...>   within f(x) do x * 1000 end\n    ...>   def f(1) do Process.sleep(10000) end\n    ...>   def f(x) do x end\n    ...> end\n    iex> Elixir.Corsa.Within.Example.f(10)\n    10\n    iex> Elixir.Corsa.Within.Example.f(1)\n    ** (Corsa.WithinViolationError) @within does not hold in call 'Corsa.Within.Example.f(1)'","ref":"Corsa.Within.html#module-example"},{"type":"macro","title":"Corsa.Within.within/2","doc":"","ref":"Corsa.Within.html#within/2"},{"type":"macro","title":"Errors - Corsa.Within.within/2","doc":"iex> defmodule Elixir.Corsa.Within.ExampleError1 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Within\n    ...>   within f(x) do x * 1000 end\n    ...>   within f(x) do x * 1000 end\n    ...>   def f(1) do Process.sleep(10000) end\n    ...>   def f(x) do x end\n    ...> end\n    ** (Corsa.WithinError) @within for function 'f/1' already defined\n\n    iex> defmodule Elixir.Corsa.Within.ExampleError2 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Within\n    ...>   within f(x, x) do x * 1000 end\n    ...>   def f(1, _) do Process.sleep(10000) end\n    ...>   def f(x, _) do x end\n    ...> end\n    ** (Corsa.WithinError) arguments in @within should contain different names\n\n    iex> defmodule Elixir.Corsa.Within.ExampleError3 do\n    ...>   use Corsa.Assert\n    ...>   use Corsa.Within\n    ...>   within f(x, _) do x * 1000 end\n    ...>   def f(1, _) do Process.sleep(10000) end\n    ...>   def f(x, _) do x end\n    ...> end\n    ** (Corsa.WithinError) arguments in @within cannot be ignored with _","ref":"Corsa.Within.html#within/2-errors"},{"type":"exception","title":"Corsa.AssertError","doc":"","ref":"Corsa.AssertError.html"},{"type":"exception","title":"Corsa.AssertViolationError","doc":"","ref":"Corsa.AssertViolationError.html"},{"type":"exception","title":"Corsa.DeclareError","doc":"","ref":"Corsa.DeclareError.html"},{"type":"exception","title":"Corsa.DeclareViolationError","doc":"","ref":"Corsa.DeclareViolationError.html"},{"type":"exception","title":"Corsa.DecreasesError","doc":"","ref":"Corsa.DecreasesError.html"},{"type":"exception","title":"Corsa.DecreasesViolationError","doc":"","ref":"Corsa.DecreasesViolationError.html"},{"type":"exception","title":"Corsa.ExpectsError","doc":"","ref":"Corsa.ExpectsError.html"},{"type":"exception","title":"Corsa.ExpectsViolationError","doc":"","ref":"Corsa.ExpectsViolationError.html"},{"type":"exception","title":"Corsa.MonitorError","doc":"","ref":"Corsa.MonitorError.html"},{"type":"exception","title":"Corsa.PostError","doc":"","ref":"Corsa.PostError.html"},{"type":"exception","title":"Corsa.PostViolationError","doc":"","ref":"Corsa.PostViolationError.html"},{"type":"exception","title":"Corsa.PreError","doc":"","ref":"Corsa.PreError.html"},{"type":"exception","title":"Corsa.PreViolationError","doc":"","ref":"Corsa.PreViolationError.html"},{"type":"exception","title":"Corsa.SpecArgViolationError","doc":"","ref":"Corsa.SpecArgViolationError.html"},{"type":"exception","title":"Corsa.SpecError","doc":"","ref":"Corsa.SpecError.html"},{"type":"exception","title":"Corsa.SpecResultViolationError","doc":"","ref":"Corsa.SpecResultViolationError.html"},{"type":"exception","title":"Corsa.StateM.InitError","doc":"","ref":"Corsa.StateM.InitError.html"},{"type":"exception","title":"Corsa.StateM.NextError","doc":"","ref":"Corsa.StateM.NextError.html"},{"type":"exception","title":"Corsa.ThrowsError","doc":"","ref":"Corsa.ThrowsError.html"},{"type":"exception","title":"Corsa.ThrowsViolationError","doc":"","ref":"Corsa.ThrowsViolationError.html"},{"type":"exception","title":"Corsa.WithinError","doc":"","ref":"Corsa.WithinError.html"},{"type":"exception","title":"Corsa.WithinViolationError","doc":"","ref":"Corsa.WithinViolationError.html"},{"type":"extras","title":"Corsa","doc":"# Corsa\n\n`Corsa` is a library to write executable code contracts in Elixir.","ref":"readme.html"},{"type":"extras","title":"Installation - Corsa","doc":"The package can be installed by adding `corsa` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:corsa, git: \"git@gitlab.com:babel-upm/corsa.git\", branch: \"main\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Contracts - Corsa","doc":"Corsa contracts are written using normal pre- and post-condition contracts familiar from other\nprogramming languages and libraries such as Eiffel. To use the library in a module we must first\ninsert a\n\n```elixir\nuse Corsa\n```\n\nstatement in the module. Suppose next that we have implemented a function `list_to_nat` which\nconverts an integer represented as a list of small numbers (between 0 and 9) to an integer:\n\nSuppose next that we have implemented a function `list_to_nat` which converts an integer\nrepresented as a list of small numbers (between 0 and 9) to an integer. We can require that the\nargument to `list_to_nat` is correct by specifying a `@pre` contract:\n\n```elixir\niex> defmodule Example do\n...>   use Corsa\n...>   @pre list_to_nat(l), do: is_decimal_list(l)\n...>   @post list_to_nat(l), do: is_integer(result)\n...>   def list_to_nat(list), do: list_to_nat(list, 0)\n...>   defp list_to_nat([], result), do: result\n...>   defp list_to_nat([ n| rest ], acc) do\n...>   list_to_nat(rest ,10*acc+n)\n...>   end\n...>   defp is_decimal_list([n|rest]) do\n...>     is_decimal(n) and (n > 0)\n...>     and Enum.all?(rest,&is_decimal/1)\n...>   end\n...>   defp is_decimal_list(_), do: false\n...>   defp is_decimal(n) do\n...>     is_integer(n) and (n >= 0) and (n < 10)\n...>   end\n...> end\niex> Example.list_to_nat([1,5])\n15\niex> Example.list_to_nat(5)\n** (Corsa.PreViolationError) @pre does not hold in call 'CorsaDoctestTest.Example.list_to_nat(5)'\n```\n\nIf we now invoke the function with an incorrect argument by default an informative exception is\nraised.\n\nSimilarly `@post` contracts check that the value returned by a function is correct:","ref":"readme.html#contracts"},{"type":"extras","title":"Assertions - Corsa","doc":"Assertions can be stated using the `@assert` construct:\n\n```elixir\niex> defmodule Example do\n...>   use Corsa\n...>   def f(y) do\n...>     x = g(y)\n...>     @assert x > 0\n...>     x + 10\n...>   end\n...>   defp g(x) do x end\n...> end\niex> Example.f(10)\n20\niex> Example.f(-10)\n** (Corsa.AssertViolationError) @assert does not hold in expression 'x > 0' with values: 'x = -10'\n```","ref":"readme.html#assertions"},{"type":"extras","title":"Termination Guarantees - Corsa","doc":"Corsa can check whether recursive functions eventually terminate, through specifying a measure\nusing the `@decreases` construct:\n\n```elixir\niex> defmodule Example do\n...>   use Corsa\n...>   @decreases sum(l), do: length(l)\n...>   def sum([]), do: 0\n...>   def sum([n|rest]), do: n+sum(rest)\n...> end\niex> Example.sum([1,2,3])\n6\n```\n\nThe `@decreases` annotation specifies a function, that when applied to the arguments of the called\nfunction, calculates a measure (e.g., an integer). Corsa checks that measure when a function is\ninvoked is strictly greater than the measure in recursive calls.  In the example above, the\nmeasure used is the length of the least which strictly decreases in recursive calls.  The check\nwill be made even if the recursive call is made within another function.\n\nNote, however, that a decreasing measure does not necessarily guarantee termination:\n\n```elixir\ndefmodule Example do\n  use Corsa\n  @decreases fact(n), do: n\n  def fact(1), do: 1\n  def fact(n), do: n * fact(n - 1)\nend\n```\n\nIf the function is invoked with `fact(-1)` then it obviously never terminates (except possibly\nwith an exception) but the measure always strictly decreases. Note you can \"fix\" the previous\ncontract by using `abs/1`:\n\n```elixir\niex> defmodule Example do\n...>   use Corsa\n...>   @decreases fact(n), do: abs(n)\n...>   def fact(1), do: 1\n...>   def fact(n), do: n * fact(n - 1)\n...> end\niex> Example.fact(10)\n3628800\niex> Example.fact(0)\n** (Corsa.DecreasesViolationError) @decreases does not hold, call 'CorsaDoctestTest.Example.fact(-1)' does not decrease after 'CorsaDoctestTest.Example.fact(0)'\n```","ref":"readme.html#termination-guarantees"},{"type":"extras","title":"Time Contracts - Corsa","doc":"Time bounds for long a function application may execute can be specified with the `@within`\nconstruct:\n\n```elixir\ndefmodule Example do\n  use Corsa\n  @within fact(n), do: 2000\n  def fact(1), do: 1\n  def fact(n), do: n*fact(n-1)\nend\n```\n\nIf the call to the factorial function does not return an answer within 2000 milliseconds, an error\nis logged.","ref":"readme.html#time-contracts"},{"type":"extras","title":"Message Passing Contracts - Corsa","doc":"The `@expects` contract specifies the accepatble messages in a receive statement.\n\n```elixir\niex> defmodule Example do\n...>   use Corsa\n...>\n...>   def counter(state) do\n...>     @expects [v > 0, receive do\n...>       :inc -> counter(state + 1)\n...>       :dec -> counter(state - 1)\n...>       {:set, v} -> counter(v)\n...>     end]\n...>   end\n...> end\niex> send(self(), :inc)\niex> send(self(), {:set, 10})\niex> send(self(), {:set, -1})\niex> Example.counter(0)\n** (Corsa.ExpectsViolationError) @expects does not hold in expression 'v > 0' with values: 'v = -1'\n```","ref":"readme.html#message-passing-contracts"},{"type":"extras","title":"Type Contracts - Corsa","doc":"When using `Corsa` type annotations will be used to generate runtime typechecks for your\nfunctions.\n\n```elixir\niex> defmodule Example do\n...>   use Corsa\n...>\n...>   @spec id(integer()) :: integer()\n...>   @spec id(boolean()) :: boolean()\n...>   def id(x) do x end\n...> end\niex> Example.id(10)\n10\niex> Example.id(true)\ntrue\niex> Example.id(\"hello\")\n** (Corsa.SpecArgViolationError) @spec does not hold in arguments in call 'CorsaDoctestTest.Example.id(\"hello\")'\n```","ref":"readme.html#type-contracts"},{"type":"extras","title":"Configuring Corsa - Corsa","doc":"Corsa can be configured to either (the default) raise exceptions whenever a contract is violated,\nor log contract violations, or do nothing.\n\nTo disable Corsa:\n\n```elixir\ndefmodule Example do\n  use Corsa, disabled: true\n  ...\nend\n```\n\nTo log contracts, you must specify the logging level:\n\n```elixir\ndefmodule Example do\n  use Corsa, logger: :error\n  ...\nend\n```\n\nWhen logging is enabled, contracts can be executed asynchronously. To enable asynchronous execution:\n\n```elixir\ndefmodule Example do\n  use Corsa, logger: :error, async: true\nend\n```\n\nGlobal configuration can be performed through the configuration file located at `config/config.exs`:\n\n```elixir\nconfig :corsa, disabled: true\n```","ref":"readme.html#configuring-corsa"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}